<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
namespace Montonio\Hirepurchase\Block\Adminhtml\System\Config\Form\Field;

use Montonio\Hirepurchase\Model\Payment\SplitConfigProvider;

/**
 * @inheritdoc
 */
class AvailableSplitMethods extends \Magento\Config\Block\System\Config\Form\Field\FieldArray\AbstractFieldArray
{
    /**
     * Styles to apply to disabled field to allow field save.
     */
    private const DISABLED_FIELD_STYLES = [
        'width: 50px',
        'pointer-events: none',
        'background-color: #e9e9e9',
        'border-color: #adadad',
        'color: #303030',
        'opacity: .5',
        'cursor: not-allowed',
    ];

    /**
     *
     * @param \Montonio\Hirepurchase\Gateway\Split\Command\AspspsCommand $aspspsCommand
     * @param \Magento\Framework\View\Element\BlockFactory $blockFactory
     * @param \Magento\Framework\Data\Form\Element\Factory $elementFactory
     * @param \Magento\Framework\View\Design\Theme\LabelFactory $labelFactory
     * @param \Magento\Backend\Block\Template\Context $context
     * @param array $data
     */
    public function __construct(
        \Montonio\Hirepurchase\Gateway\Split\Command\AspspsCommand $aspspsCommand,
        \Magento\Framework\View\Element\BlockFactory $blockFactory,
        \Magento\Framework\Data\Form\Element\Factory $elementFactory,
        \Magento\Framework\View\Design\Theme\LabelFactory $labelFactory,
        \Magento\Backend\Block\Template\Context $context,
        array $data = []
    ) {

        $this->aspspsCommand = $aspspsCommand;
        $this->blockFactory = $blockFactory;
        $this->elementFactory = $elementFactory;
        $this->labelFactory = $labelFactory;

        $this->setTemplate('Montonio_Hirepurchase::system/config/form/field/array.phtml');

        parent::__construct($context, $data);
    }

    public function getArrayRows()
    {
        $values = $this->getElement()->getValue() ?? [];

        try {
            $methods = $this->retrieveApiData();

            foreach ($methods as $key => $method) {
                $values[$key] = $values[$key] ?? $method;
            }

        } catch (\Exception $e) {
            //silence
            $this->silence();
        }

        $this->getElement()->setValue($values);

        return parent::getArrayRows();
    }

    /**
     * Sort array by field.
     *
     * @param array $array
     * @param string $field
     *
     * @return array
     */
    public function sortArrayByField(array $array, string $field): array
    {
        $output = [];
        $undefined = [];

        foreach ($array as $data) {
            if (empty($data[$field])) {
                $undefined['undefined'][] = $data;
            } else {
                $output[$data[$field]][] = $data;
            }
        }

        return $undefined + $output;
    }

    /**
     * Retrieve payment methods API data.
     *
     * @return array
     */
    private function retrieveApiData(): array
    {
        $output = [];
        $methods = $this->aspspsCommand->execute([]);
        foreach ($methods as $country => $methodsData) {

            $sort = SplitConfigProvider::DEFAULT_SORT_ORDER;

            foreach ($methodsData as $method) {
                if (\strtolower($method['bic']) === SplitConfigProvider::CARD_PAYMENT_BIC) {
                    $country = null;
                }

                $output[$method['bic']] = [
                    'method_country' => $country,
                    'method_enabled' => '0',
                    'method_title' => $method['name'],
                    'method_desc' => $method['name'],
                    'logo_url' => $method['logo_url'],
                    'sort_order' => $sort
                ];

                $sort = $sort + 10;
            }
        }

        return $output;
    }

    /**
     * Prepare to render
     *
     * @return void
     */
    public function _prepareToRender(): void
    {
        $this->addColumn('method_country', [
            'label' => __('Country'),
            'style' => \implode(';', self::DISABLED_FIELD_STYLES),
        ]);

        $this->addColumn('method_enabled', [
            'label' => __('On/Off'),
            'style' => 'width: 15px'
        ]);

        $this->addColumn('method_title', [
            'label' => __('Title'),
            'style' => 'width: 150px'
        ]);

        $this->addColumn('method_desc', [
            'label' => __('Description'),
            'style' => 'width: 150px'
        ]);

        $this->addColumn('logo_url', [
            'label' => __('Logo Url'),
            'style' => 'width: 150px'
        ]);

        $this->addColumn('sort_order', [
            'label' => __('Sort Order'),
            'style' => 'width: 45px',
            'class' => 'validate-number',
        ]);

        $this->_addAfter = false;
        $this->_addButtonLabel = __('Add Fields');
    }

    /**
     * Render array cell for prototypeJS template
     *
     * @param string $columnName
     * @return string
     */
    public function renderCellTemplate($columnName)
    {
        if ($columnName == 'method_enabled' && isset($this->_columns[$columnName])) {
            /** @var $label \Magento\Framework\View\Design\Theme\Label */
            $label = $this->labelFactory->create();
            $options = ['0' => __('Off'), '1' => __('On')];
            $element = $this->elementFactory->create('select');
            $element->setForm($this->getForm())
                ->setName($this->_getCellInputElementName($columnName))
                ->setHtmlId($this->_getCellInputElementId('<%- _id %>', $columnName))
                ->setValues($options);
            return str_replace("\n", '', $element->getElementHtml());
        }

        return parent::renderCellTemplate($columnName);
    }

    /**
     * Do nothing
     * @return void
     */
    public function silence() : void
    {
    }
}
