<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
declare(strict_types=1);
namespace Montonio\Hirepurchase\Gateway\Payments\Config;

use \Firebase\JWT\JWT;

/**
 * @inheritdoc
 */
class Config extends \Magento\Payment\Gateway\Config\Config
{

    /**
     *
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Framework\Escaper $escaper
     * @param \Magento\Framework\Serialize\Serializer\Json $json
     * @param string $methodCode
     * @param string $pathPattern
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Framework\Escaper $escaper,
        \Magento\Framework\Serialize\Serializer\Json $json,
        \Psr\Log\LoggerInterface $logger,
        $methodCode = null,
        $pathPattern = self::DEFAULT_PATH_PATTERN
    ) {
        $this->scopeConfig = $scopeConfig;
        $this->escaper = $escaper;
        $this->json = $json;
        $this->logger = $logger;

        //if we DI this class directly to our other components we need this to be initiated
        //in all other case it is initiated trough di.xml
        if (!$methodCode) {
            $methodCode = \Montonio\Hirepurchase\Model\Payment\PaymentsConfigProvider::CODE;
        }

        $this->methodCode = $methodCode;

        parent::__construct($scopeConfig, $methodCode, $pathPattern);
    }

    /**
     * @return bool
     */
    public function isActive($storeId = null)
    {
        return $this->getValue('active', $storeId);
    }

    /**
     *
     * @return string
     */
    public function getCode()
    {
        return $this->methodCode;
    }

    /**
     *
     * @return string
     */
    public function getServiceDescription()
    {
        return $this->escaper->escapeHtml($this->getValue('service_description'));
    }

    /**
     * Retrieve title display option.
     *
     * @return string
     */
    public function getTitleDisplayOption(): string
    {
        return $this->escaper->escapeHtml((string)$this->getValue('title_display_option'));
    }

    /**
     *
     * @return string
     */
    public function getServiceLogoPath()
    {
        return $this->escaper->escapeHtml($this->getValue('service_logo_path'));
    }

    /**
     * Retrieve assoc array of checkout configuration
     *
     * @return array
     */
    public function getConfig()
    {
        //we do not use this and return empty array
        return [];
    }

    /**
     *
     * @param mixed $data
     * @param string $key
     * @return string
     */
    public function generateSignature($data, $key, $jsonEncode = false)
    {
        if ($jsonEncode) {
            $data = json_encode($data);
        }
        return JWT::encode($data, $key);
    }

    /**
     *
     * @param string $data
     * @param string $key
     * @return array|string
     */
    public function decodeSignature($data, $key)
    {
        $allowed_algs = [
            'HS256'
        ];
        return JWT::decode($data, $key, $allowed_algs);
    }

    /**
     *
     * @param array $data
     * @param string $accesKey
     * @return boolean
     */
    public function validateSignedData($data, $accesKey)
    {
        $result = false;

        //keys are defined in config.xml
        $requiredKeys = explode(',', $this->getValue('validate_fields'));

        $dataValidator = [];
        foreach ($requiredKeys as $k => $v) {
            if (!isset($data[$v])) {
                $dataValidator[$v] = false;
                $this->logger->critical(__('Payment token is invalid, missing %1 from response message', $v));
            }
        }

        if (count($dataValidator) === 0) {
            $result = true;
        }

        if ($data['access_key'] !== $accesKey) {
            $result = false;
        }

        return $result;
    }

    /**
     *
     */
    public function getAvailableMethods()
    {

        $methods = [];
        try {
            $values = $this->json->unserialize($this->getValue('available_methods'));
            $methods = array_flip(array_keys($values));

            foreach ($methods as $key => $value) {
                if (is_array($values[$key])) {
                    $methods[$key]=$values[$key];
                }

                if (is_array($values[$key]) && !$values[$key]['method_enabled']) {
                    unset($methods[$key]);
                }
            }
        } catch (\Exception $e) {
            $this->logger->critical($e);
        }

        //if grouped we add this to the mix so its counted as available
        if ($this->getValue('grouped')) {
            //grouped is a imaginary code for grouped method
            $methods['grouped'] = (!empty($methods)) ? (int)count($methods) + 1 : 0;
        }
        return $methods;
    }
}
